@@ -13,6 +13,7 @@
 //---------------------------------------------------------------------------
 //
 // Copyright (c) 2000  Caldera Systems, Inc.
+// Copyright (c) 2018  Harald Sitter <sitter@kde.org>
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -30,13 +31,20 @@
 //
 /////////////////////////////////////////////////////////////////////////////
 
+#include "kio_smb.h"
+#include "kio_smb_internal.h"
+
+#include <DNSSD/ServiceBrowser>
+#include <DNSSD/RemoteService>
+#include <KLocalizedString>
+#include <KIO/Job>
+
+#include <QEventLoop>
+
 #include <pwd.h>
 #include <grp.h>
 
-#include "kio_smb.h"
-#include "kio_smb_internal.h"
-#include <KLocalizedString>
-#include <KIO/Job>
+#include <config-runtime.h>
 
 using namespace KIO;
 
@@ -69,8 +77,8 @@ int SMBSlave::browse_stat_path(const SMBUrl& _url, UDSEntry& udsentry)
          return EINVAL;
       }
 
-      udsentry.insert(KIO::UDSEntry::UDS_FILE_TYPE, st.st_mode & S_IFMT);
-      udsentry.insert(KIO::UDSEntry::UDS_SIZE, st.st_size);
+      udsentry.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE, st.st_mode & S_IFMT);
+      udsentry.fastInsert(KIO::UDSEntry::UDS_SIZE, st.st_size);
 
       QString str;
       uid_t uid = st.st_uid;
@@ -79,7 +87,7 @@ int SMBSlave::browse_stat_path(const SMBUrl& _url, UDSEntry& udsentry)
           str = user->pw_name;
       else
           str = QString::number( uid );
-      udsentry.insert(KIO::UDSEntry::UDS_USER, str);
+      udsentry.fastInsert(KIO::UDSEntry::UDS_USER, str);
 
       gid_t gid = st.st_gid;
       struct group *grp = getgrgid( gid );
@@ -87,11 +95,11 @@ int SMBSlave::browse_stat_path(const SMBUrl& _url, UDSEntry& udsentry)
           str = grp->gr_name;
       else
           str = QString::number( gid );
-      udsentry.insert(KIO::UDSEntry::UDS_GROUP, str);
+      udsentry.fastInsert(KIO::UDSEntry::UDS_GROUP, str);
 
-      udsentry.insert(KIO::UDSEntry::UDS_ACCESS, st.st_mode & 07777);
-      udsentry.insert(KIO::UDSEntry::UDS_MODIFICATION_TIME, st.st_mtime);
-      udsentry.insert(KIO::UDSEntry::UDS_ACCESS_TIME, st.st_atime);
+      udsentry.fastInsert(KIO::UDSEntry::UDS_ACCESS, st.st_mode & 07777);
+      udsentry.fastInsert(KIO::UDSEntry::UDS_MODIFICATION_TIME, st.st_mtime);
+      udsentry.fastInsert(KIO::UDSEntry::UDS_ACCESS_TIME, st.st_atime);
       // No, st_ctime is not UDS_CREATION_TIME...
    }
 
@@ -118,7 +126,7 @@ void SMBSlave::stat( const QUrl& kurl )
 
     UDSEntry    udsentry;
     // Set name
-    udsentry.insert( KIO::UDSEntry::UDS_NAME, kurl.fileName() );
+    udsentry.fastInsert( KIO::UDSEntry::UDS_NAME, kurl.fileName() );
 
     switch(m_current_url.getType())
     {
@@ -128,7 +136,7 @@ void SMBSlave::stat( const QUrl& kurl )
 
     case SMBURLTYPE_ENTIRE_NETWORK:
     case SMBURLTYPE_WORKGROUP_OR_SERVER:
-        udsentry.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
+        udsentry.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
         break;
 
     case SMBURLTYPE_SHARE_OR_PATH:
@@ -330,12 +338,15 @@ void SMBSlave::listDir( const QUrl& kurl )
    qCDebug(KIO_SMB) << "open " << m_current_url.toSmbcUrl() << " " << m_current_url.getType() << " " << dirfd;
    if(dirfd >= 0)
    {
+       uint direntCount = 0;
        do {
            qCDebug(KIO_SMB) << "smbc_readdir ";
            dirp = smbc_readdir(dirfd);
            if(dirp == nullptr)
                break;
 
+           ++direntCount;
+
            // Set name
            QString udsName;
            const QString dirpName = QString::fromUtf8( dirp->name );
@@ -352,12 +363,12 @@ void SMBSlave::listDir( const QUrl& kurl )
 
            qCDebug(KIO_SMB) << "dirp->name " <<  dirp->name  << " " << dirpName << " '" << comment << "'" << " " << dirp->smbc_type;
 
-           udsentry.insert( KIO::UDSEntry::UDS_NAME, udsName );
+           udsentry.fastInsert( KIO::UDSEntry::UDS_NAME, udsName );
 
            // Mark all administrative shares, e.g ADMIN$, as hidden. #197903
            if (dirpName.endsWith(QLatin1Char('$'))) {
               //qCDebug(KIO_SMB) << dirpName << "marked as hidden";
-              udsentry.insert(KIO::UDSEntry::UDS_HIDDEN, 1);
+              udsentry.fastInsert(KIO::UDSEntry::UDS_HIDDEN, 1);
            }
 
            if (udsName == ".")
@@ -395,11 +406,11 @@ void SMBSlave::listDir( const QUrl& kurl )
                    dirp->smbc_type == SMBC_FILE_SHARE)
            {
                // Set type
-               udsentry.insert( KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR );
+               udsentry.fastInsert( KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR );
 
 
                if (dirp->smbc_type == SMBC_SERVER) {
-                   udsentry.insert(KIO::UDSEntry::UDS_ACCESS, (S_IRUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH));
+                   udsentry.fastInsert(KIO::UDSEntry::UDS_ACCESS, (S_IRUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH));
 
                    // QString workgroup = m_current_url.host().toUpper();
                    QUrl u("smb://");
@@ -408,11 +419,11 @@ void SMBSlave::listDir( const QUrl& kurl )
                    // when libsmbclient knows
                    // u = QString("smb://%1?WORKGROUP=%2").arg(dirpName).arg(workgroup.toUpper());
                    qCDebug(KIO_SMB) << "list item " << u;
-                   udsentry.insert(KIO::UDSEntry::UDS_URL, u.url());
+                   udsentry.fastInsert(KIO::UDSEntry::UDS_URL, u.url());
 
-                   udsentry.insert(KIO::UDSEntry::UDS_MIME_TYPE, QString::fromLatin1("application/x-smb-server"));
+                   udsentry.fastInsert(KIO::UDSEntry::UDS_MIME_TYPE, QString::fromLatin1("application/x-smb-server"));
                } else
-                   udsentry.insert(KIO::UDSEntry::UDS_ACCESS, (S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH));
+                   udsentry.fastInsert(KIO::UDSEntry::UDS_ACCESS, (S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH));
 
 
                // Call base class to list entry
@@ -421,17 +432,17 @@ void SMBSlave::listDir( const QUrl& kurl )
            else if(dirp->smbc_type == SMBC_WORKGROUP)
            {
                // Set type
-               udsentry.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
+               udsentry.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
 
                // Set permissions
-               udsentry.insert(KIO::UDSEntry::UDS_ACCESS, (S_IRUSR | S_IRGRP | S_IROTH | S_IXUSR | S_IXGRP | S_IXOTH));
+               udsentry.fastInsert(KIO::UDSEntry::UDS_ACCESS, (S_IRUSR | S_IRGRP | S_IROTH | S_IXUSR | S_IXGRP | S_IXOTH));
 
-               udsentry.insert(KIO::UDSEntry::UDS_MIME_TYPE, QString::fromLatin1("application/x-smb-workgroup"));
+               udsentry.fastInsert(KIO::UDSEntry::UDS_MIME_TYPE, QString::fromLatin1("application/x-smb-workgroup"));
 
                // QString workgroup = m_current_url.host().toUpper();
                QUrl u("smb://");
                u.setHost(dirpName);
-               udsentry.insert(KIO::UDSEntry::UDS_URL, u.url());
+               udsentry.fastInsert(KIO::UDSEntry::UDS_URL, u.url());
 
                // Call base class to list entry
                listEntry(udsentry);
@@ -447,14 +458,16 @@ void SMBSlave::listDir( const QUrl& kurl )
            udsentry.clear();
        } while (dirp); // checked already in the head
 
+       listDNSSD(udsentry, url, direntCount);
+
        if (dir_is_root) {
-           udsentry.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
-           udsentry.insert(KIO::UDSEntry::UDS_NAME, ".");
-           udsentry.insert(KIO::UDSEntry::UDS_ACCESS, (S_IRUSR | S_IRGRP | S_IROTH | S_IXUSR | S_IXGRP | S_IXOTH));
+           udsentry.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
+           udsentry.fastInsert(KIO::UDSEntry::UDS_NAME, ".");
+           udsentry.fastInsert(KIO::UDSEntry::UDS_ACCESS, (S_IRUSR | S_IRGRP | S_IROTH | S_IXUSR | S_IXGRP | S_IXOTH));
        }
        else
        {
-           udsentry.insert(KIO::UDSEntry::UDS_NAME, ".");
+           udsentry.fastInsert(KIO::UDSEntry::UDS_NAME, ".");
            const int statErr = browse_stat_path(m_current_url, udsentry);
            if (statErr)
            {
@@ -463,8 +476,8 @@ void SMBSlave::listDir( const QUrl& kurl )
                    reportWarning(m_current_url, statErr);
                }
                // Create a default UDSEntry if we could not stat the actual directory
-               udsentry.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
-               udsentry.insert(KIO::UDSEntry::UDS_ACCESS, (S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH));
+               udsentry.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
+               udsentry.fastInsert(KIO::UDSEntry::UDS_ACCESS, (S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH));
            }
        }
        listEntry(udsentry);
@@ -490,6 +503,116 @@ void SMBSlave::listDir( const QUrl& kurl )
    finished();
 }
 
+void SMBSlave::listDNSSD(UDSEntry &udsentry, const QUrl &url, const uint direntCount)
+{
+    // Certain versions of KDNSSD suffer from signal races which can easily
+    // deadlock the slave.
+#ifndef HAVE_KDNSSD_WITH_SIGNAL_RACE_PROTECTION
+    return;
+#endif // HAVE_KDNSSD_WITH_SIGNAL_RACE_PROTECTION
+
+    // This entire method act as fallback logic iff SMB discovery is not working
+    // (for example when using a protocol version that doesn't have discovery).
+    // As such we can return if entries were discovered or the URL is not '/'
+    auto normalizedUrl = url.adjusted(QUrl::NormalizePathSegments);
+    if (direntCount > 0 || !normalizedUrl.path().isEmpty()) {
+        return;
+    }
+
+    // Slaves have no event loop, start one for the poll.
+    // KDNSSD has an internal timeout which may trigger if this takes too long
+    // so in theory this should not ever be able to get stuck.
+    // The eventloop runs until the discovery is finished. The finished slot
+    // will quit it.
+    QList<KDNSSD::RemoteService::Ptr> services;
+    QEventLoop e;
+    KDNSSD::ServiceBrowser browser(QStringLiteral("_smb._tcp"));
+    connect(&browser, &KDNSSD::ServiceBrowser::serviceAdded,
+            this, [&services](KDNSSD::RemoteService::Ptr service){
+        qCDebug(KIO_SMB) << "DNSSD added:"
+                         << service->serviceName()
+                         << service->type()
+                         << service->domain()
+                         << service->hostName()
+                         << service->port();
+        // Manual contains check. We need to use the == of the underlying
+        // objects, not the pointers. The same service may have >1
+        // RemoteService* instances representing it, so the == impl of
+        // RemoteService::Ptr is useless here.
+        for (const auto &it : services) {
+            if (*service == *it) {
+                return;
+            }
+        }
+        // Schedule resolution of hostname. We'll later call resolve
+        // which will block until the resolution is done. This basically
+        // gives us a head start.
+        service->resolveAsync();
+        services.append(service);
+    });
+    connect(&browser, &KDNSSD::ServiceBrowser::serviceRemoved,
+            this, [&services](KDNSSD::RemoteService::Ptr service){
+        qCDebug(KIO_SMB) << "DNSSD removed:"
+                         << service->serviceName()
+                         << service->type()
+                         << service->domain()
+                         << service->hostName()
+                         << service->port();
+        services.removeAll(service);
+    });
+    connect(&browser, &KDNSSD::ServiceBrowser::finished,
+            this, [&]() {
+        browser.disconnect(); // Stop sending anything, we'll exit here.
+        // Resolution still requires an event loop. So, let the resolutions
+        // finish and then quit the loop. Services that fail resolution
+        // get dropped since we won't be able to access them properly.
+        for (auto it = services.begin(); it != services.end(); ++it) {
+            auto service = *it;
+            if (!service->resolve()) {
+                qCWarning(KIO_SMB) << "Failed to resolve DNSSD service"
+                                   << service->serviceName();
+                it = services.erase(it);
+            }
+        }
+        e.quit();
+    });
+    browser.startBrowse();
+    e.exec();
+
+    for (const auto &service : services) {
+        udsentry.fastInsert(KIO::UDSEntry::UDS_NAME, service->serviceName());
+
+        udsentry.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
+        udsentry.fastInsert(KIO::UDSEntry::UDS_ACCESS, (S_IRUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH));
+
+        // TODO: it may be better to resolve the host to an ip address. dnssd
+        //   being able to find a service doesn't mean name resolution is
+        //   properly set up for its domain. So, we may not be able to resolve
+        //   this without help from avahi. OTOH KDNSSD doesn't have API for this
+        //   and from a platform POV we should probably assume that if avahi
+        //   is functional it is also set up as resolution provider.
+        //   Given the plugin design on glibc's libnss however I am not sure
+        //   that assumption will be true all the time. ~sitter, 2018
+        QUrl u(QStringLiteral("smb://"));
+        u.setHost(service->hostName());
+        if (service->port() > 0 && service->port() != 445 /* default smb */) {
+            u.setPort(service->port());
+        }
+
+        udsentry.fastInsert(KIO::UDSEntry::UDS_URL, u.url());
+        udsentry.fastInsert(KIO::UDSEntry::UDS_MIME_TYPE,
+                            QStringLiteral("application/x-smb-server"));
+
+        listEntry(udsentry);
+        udsentry.clear();
+    }
+
+    // NOTE: workgroups cannot be properly resolved because libsmbclient
+    //   seems to lack the appropriate API to pull this data out of netbios.
+    //   Netbios is also not working on IPv6 and its replacement (LLMNR)
+    //   doesn't support the concept of workgroups.
+}
+
 void SMBSlave::fileSystemFreeSpace(const QUrl& url)
 {
     qCDebug(KIO_SMB) << url;
